const fs = require("fs");
const { Transform } = require("stream");
const excel = require("@adncorp/excel-stream");
const request = require("request");
const validUrl = require("valid-url");

function getReadStream(file) {
  if (validUrl.isUri(file)) {
    return request(file);
  }
  return fs.createReadStream(file);
}

function ExcelParser() {}

ExcelParser.prototype.parse = function parse(
  file,
  callbackFn,
  batchSize,
  transformFn,
  timeOut
) {
  const bSize = batchSize && typeof batchSize === "number" ? batchSize : 10000;
  // stream of input file
  const stream = getReadStream(file);
  const tOut = timeOut && typeof timeOut === "number" ? timeOut : 100;
  // convert excel to object stream
  stream
    .pipe(excel({ sheetIndex: 0 }))
    //process object stream and return formated object for collbackFn
    .pipe(getTransformObject(callbackFn, bSize, transformFn, tOut));
};

// return transform stream which will do proccessing
function getTransformObject(callbackFn, batchSize, transformFn, timeOut) {
  const jsonToDb = new Transform({
    writableObjectMode: true,
    readableObjectMode: true,
    transform(chunk, encoding, callback) {
      // console.log(
      //   `The script uses approximately ${Math.round(
      //     process.memoryUsage().heapUsed / 1024 / 1024 * 100
      //   ) / 100} MB`
      // );
      this.records.push(Object.values(chunk));
      // batch processing of records
      if (this.records.length === batchSize) {
        applyCustomTransformation(
          this.records,
          transformFn,
          timeOut
        ).then(data => {
          // console.log(
          //   `The script uses approximately ${Math.round(
          //     process.memoryUsage().heapUsed / 1024 / 1024 * 100
          //   ) / 100} MB`
          // );
          data.map(record => callbackFn(record));
          this.records = [];
          callback();
        });
      } else {
        callback();
      }
    },
    flush(done) {
      // flush we repeat steps for last records,
      // eg total records 108, last 8 records are left to process
      if (this.records.length > 0) {
        applyCustomTransformation(
          this.records,
          transformFn,
          timeOut
        ).then(data => {
          data.map(record => callbackFn(record));
          this.records = [];
          done();
        });
      } else {
        done();
      }
    }
  });
  jsonToDb.records = [];
  return jsonToDb;
}

// async function to process data
function applyCustomTransformation(elements, transformFn, timeOut) {
  return new Promise((resolve, reject) => {
    setTimeout(() => {
      resolve(
        elements.map(
          element =>
            typeof transformFn === "function" ? transformFn(element) : element
        )
      );
    }, timeOut);
  });
}

module.exports = ExcelParser;
